"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
// import Translator from "../constants/strings/translator";
// import StringNamespaces from "../constants/strings/StringNamespaces";
const Logger_1 = require("../Logger");
const ProjectState_1 = require("../microclimate/project/ProjectState");
const CommandUtil_1 = require("./CommandUtil");
const MCLogManager_Old_1 = require("../microclimate/project/logs/deprecated/MCLogManager-Old");
const translator_1 = require("../constants/strings/translator");
const StringNamespaces_1 = require("../constants/strings/StringNamespaces");
// const STRING_NS = StringNamespaces.LOGS;
function showAllLogs(project) {
    return __awaiter(this, void 0, void 0, function* () {
        return manageLogsInner(project, "show");
    });
}
exports.showAllLogs = showAllLogs;
function hideAllLogs(project) {
    return __awaiter(this, void 0, void 0, function* () {
        return manageLogsInner(project, "hide");
    });
}
exports.hideAllLogs = hideAllLogs;
function manageLogs(project) {
    return __awaiter(this, void 0, void 0, function* () {
        return manageLogsInner(project);
    });
}
exports.manageLogs = manageLogs;
function manageLogsInner(project, all) {
    return __awaiter(this, void 0, void 0, function* () {
        if (project == null) {
            const selected = yield CommandUtil_1.promptForProject(...ProjectState_1.default.getEnabledStates());
            if (selected == null) {
                // user cancelled
                Logger_1.default.d("User cancelled project prompt");
                return;
            }
            project = selected;
        }
        if (project.logManager instanceof MCLogManager_Old_1.default) {
            // We have to use the deprecated API
            return manageLogsDeprecated(project, all);
        }
        // Wait for the logmanager to initialize, just in case it hasn't finished yet
        yield project.logManager.initPromise;
        const logs = project.logManager.logs;
        if (logs.length === 0) {
            vscode.window.showWarningMessage("This project does not have any logs available at this time.");
            return;
        }
        if (all === "show") {
            Logger_1.default.d("Showing all logs for " + project.name);
            project.logManager.logs.forEach((log) => log.showOutput());
            yield project.logManager.toggleLogStreaming(true);
            return;
        }
        else if (all === "hide") {
            Logger_1.default.d("Hiding all logs for " + project.name);
            project.logManager.logs.forEach((log) => log.removeOutput());
            yield project.logManager.toggleLogStreaming(false);
            return;
        }
        const options = {
            canPickMany: true,
            placeHolder: "Select the logs you wish to see in the Output view"
        };
        // https://github.com/Microsoft/vscode/issues/64014
        const logsToShow = yield vscode.window.showQuickPick(logs, options);
        if (logsToShow != null) {
            // Log.d("selection", selection);
            logs.forEach((log) => {
                if (logsToShow.includes(log)) {
                    log.showOutput();
                }
                else {
                    log.removeOutput();
                }
            });
            // stop the stream if 0 logs are to be shown,
            // or restart the stream if at least one is to be shown (in case one of the ones to be shown is a new one)
            yield project.logManager.toggleLogStreaming(logsToShow.length !== 0);
        }
    });
}
const STRING_NS = StringNamespaces_1.default.CMD_OPEN_LOG;
// can't wait to delete this!
function manageLogsDeprecated(project, all) {
    return __awaiter(this, void 0, void 0, function* () {
        const options = {
            canPickMany: true
        };
        const appLogName = "Application log";
        const buildLogName = "Build log";
        const logsAvailable = [{ label: appLogName }, { label: buildLogName }];
        let toShow;
        if (all === "show") {
            toShow = [appLogName, buildLogName];
        }
        else if (all === "hide") {
            toShow = [];
        }
        else {
            const logsSelected = yield vscode.window.showQuickPick(logsAvailable, options);
            if (logsSelected == null) {
                // cancelled
                return;
            }
            toShow = logsSelected.map((qpi) => qpi.label);
        }
        const logManager = project.logManager;
        if (toShow.includes(appLogName)) {
            if (!project.state.isEnabled) {
                // If we were to create an app log for a disabled project,
                // it would just say "waiting for Microclimate to send logs" until the app starts.
                vscode.window.showErrorMessage(translator_1.default.t(STRING_NS, "noLogsForDisabled"));
            }
            else {
                if (!project.state.isStarted) {
                    vscode.window.showWarningMessage(translator_1.default.t(STRING_NS, "projectIsNotStarted", { projectName: project.name, projectState: project.state.appState }));
                }
                logManager.getOrCreateAppLog(project.id, project.name).showOutputChannel();
            }
        }
        else {
            const existingAppLog = logManager.getAppLog(project.id);
            if (existingAppLog != null) {
                existingAppLog.destroy();
            }
        }
        if (toShow.includes(buildLogName)) {
            if (!project.type.providesBuildLog) {
                vscode.window.showErrorMessage(translator_1.default.t(STRING_NS, "noBuildLogsForType", { projectType: project.type.type }));
            }
            else {
                logManager.getOrCreateBuildLog(project.id, project.name).showOutputChannel();
            }
        }
        else {
            const existingBuildLog = logManager.getBuildLog(project.id);
            if (existingBuildLog != null) {
                existingBuildLog.destroy();
            }
        }
    });
}
//# sourceMappingURL=ManageLogsCmd.js.map