"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const request = require("request-promise-native");
const Project_1 = require("../project/Project");
const Endpoints_1 = require("../../constants/Endpoints");
const MCSocket_1 = require("./MCSocket");
const ConnectionManager_1 = require("./ConnectionManager");
const Logger_1 = require("../../Logger");
const translator_1 = require("../../constants/strings/translator");
const StringNamespaces_1 = require("../../constants/strings/StringNamespaces");
const MCEnvironment_1 = require("./MCEnvironment");
class Connection {
    // public readonly detail?: string;
    constructor(mcUri, host, version, socketNS, workspacePath_) {
        this.mcUri = mcUri;
        this.host = host;
        this.version = version;
        this.socketNS = socketNS;
        this.hasConnected = false;
        // Is this connection CURRENTLY connected to its Microclimate instance
        this._isConnected = false;
        this._projects = [];
        this.needProjectUpdate = true;
        this.onConnect = () => __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d(`${this} onConnect`);
            if (this._isConnected) {
                // we already know we're connected, nothing to do until we disconnect
                return;
            }
            if (!(yield ConnectionManager_1.default.instance.verifyReconnect(this))) {
                Logger_1.default.i(`Connection has changed on reconnect! ${this} is no longer a valid Connection`);
                // this connection gets destroyed
                return;
            }
            if (this.hasConnected) {
                // things to do on reconnect, but not initial connect, go here
                this._projects.forEach((p) => p.onConnectionReconnect());
            }
            this.hasConnected = true;
            this._isConnected = true;
            Logger_1.default.d(`${this} is now connected`);
            yield this.forceUpdateProjectList();
            this.onChange();
        });
        this.onDisconnect = () => __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d(`${this} onDisconnect`);
            if (!this._isConnected) {
                // we already know we're disconnected, nothing to do until we reconnect
                return;
            }
            this._isConnected = false;
            this._projects.forEach((p) => p.onConnectionDisconnect());
            this._projects = [];
            Logger_1.default.d(`${this} is now disconnected`);
            this.onChange();
        });
        this.socket = new MCSocket_1.default(this, socketNS);
        this.workspacePath = vscode.Uri.file(workspacePath_);
        this.versionStr = MCEnvironment_1.default.getVersionAsString(version);
        // QuickPickItem
        this.label = translator_1.default.t(StringNamespaces_1.default.TREEVIEW, "connectionLabel", { uri: this.mcUri });
        // this.description = this.workspacePath.fsPath.toString();
        Logger_1.default.i(`Created new Connection @ ${this}, workspace ${this.workspacePath}`);
    }
    dispose() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d("Destroy connection " + this);
            return Promise.all([
                this.socket.dispose(),
                this._projects.map((p) => p.dispose()),
            ])
                .then(() => Promise.resolve());
        });
    }
    toString() {
        return `${this.mcUri} ${this.versionStr}`;
    }
    /**
     * Call this whenever the tree needs to be updated - ie when this connection or any of its projects changes.
     */
    onChange(changed) {
        return __awaiter(this, void 0, void 0, function* () {
            if (changed == null) {
                changed = this;
            }
            // Log.d(`Connection ${this.mcUri} changed`);
            ConnectionManager_1.default.instance.onChange(changed);
        });
    }
    get isConnected() {
        return this._isConnected;
    }
    get projects() {
        return this._projects;
    }
    updateProjects() {
        return __awaiter(this, void 0, void 0, function* () {
            // Log.d("getProjects");
            if (!this.needProjectUpdate) {
                return this._projects;
            }
            Logger_1.default.d(`Updating projects list from ${this}`);
            const projectsUrl = Endpoints_1.EndpointUtil.resolveMCEndpoint(this, Endpoints_1.MCEndpoints.PROJECTS);
            const result = yield request.get(projectsUrl, { json: true });
            const oldProjects = this._projects;
            this._projects = [];
            for (const projectInfo of result) {
                // This is a hard-coded exception for a Microclimate bug, where projects get stuck in the Deleting or Validating state
                // and don't go away until they're deleted from the workspace and MC is restarted.
                if (projectInfo.action === "deleting" || projectInfo.action === "validating") { // non-nls
                    Logger_1.default.e("Project is in a bad state and won't be displayed:", projectInfo);
                    continue;
                }
                let project;
                // If we already have a Project object for this project, just update it, don't make a new object
                // (since then the old object will go stale while code might still be referencing it)
                const existing = oldProjects.find((p) => p.id === projectInfo.projectID);
                if (existing != null) {
                    project = existing;
                    existing.update(projectInfo);
                    // Log.d("Reuse project " + project.name);
                }
                else {
                    project = new Project_1.default(projectInfo, this);
                    Logger_1.default.d("New project " + project.name);
                }
                this._projects.push(project);
            }
            this.needProjectUpdate = false;
            Logger_1.default.d("Done projects update");
            this.onChange();
            return this._projects;
        });
    }
    getProjectByID(projectID) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = this._projects.find((project) => project.id === projectID);
            if (result == null) {
                // Logger.logE(`Couldn't find project with ID ${projectID} on connection ${this}`);
            }
            return result;
        });
    }
    forceUpdateProjectList(wipeProjects = false) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d("forceUpdateProjectList");
            if (wipeProjects) {
                Logger_1.default.d(`Connection ${this} wiping ${this._projects.length} projects`);
                this._projects.forEach((p) => p.dispose());
                this._projects = [];
            }
            this.needProjectUpdate = true;
            yield this.updateProjects();
            if (wipeProjects) {
                // refresh whole tree
                this.onChange();
            }
        });
    }
    /**
     * Microclimate 1905 added:
     * - multi-logs
     * - project settings
     * - app monitor enablement
     * - new project types endpoint, for creation
     * Remove this once we drop backwards compatibility for earlier versions.
     */
    is1905OrNewer() {
        return this.version >= 1905;
    }
}
exports.default = Connection;
//# sourceMappingURL=Connection.js.map