"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const MCUtil = require("../../MCUtil");
const ProjectState_1 = require("./ProjectState");
const Logger_1 = require("../../Logger");
const translator_1 = require("../../constants/strings/translator");
const StringNamespaces_1 = require("../../constants/strings/StringNamespaces");
const ProjectOverviewPage_1 = require("./ProjectOverviewPage");
const StartModes_1 = require("../../constants/StartModes");
const MCLogManager_1 = require("./logs/MCLogManager");
const DebugUtils_1 = require("./DebugUtils");
const ProjectType_1 = require("./ProjectType");
const ProjectPendingRestart_1 = require("./ProjectPendingRestart");
const MCLogManager_Old_1 = require("./logs/deprecated/MCLogManager-Old");
const SocketEvents_1 = require("../connection/SocketEvents");
const STRING_NS = StringNamespaces_1.default.PROJECT;
class Project {
    constructor(projectInfo, connection) {
        this.connection = connection;
        // in MS
        this.RESTART_TIMEOUT = 180 * 1000;
        /**
         * Set this project's status based on the project info event payload passed.
         * This includes checking the appStatus, buildStatus, buildStatusDetail, and startMode.
         * Also updates the appPort and debugPort.
         */
        this.update = (projectInfo, isRestart = false) => {
            if (projectInfo.projectID !== this.id) {
                // shouldn't happen, but just in case
                Logger_1.default.e(`Project ${this.id} received status update request for wrong project ${projectInfo.projectID}`);
                // return the old state
                return this._state;
            }
            // Whether or not this update call has changed the project such that we have to update the UI.
            let changed = false;
            if (!isRestart) {
                // Ignore these if it's a restart because the restart event won't have them
                changed = this.setContainerID(projectInfo.containerId) || changed;
                changed = this.setLastBuild(projectInfo.lastbuild) || changed;
                // appImageLastBuild is a string
                changed = this.setLastImgBuild(Number(projectInfo.appImageLastBuild)) || changed;
                changed = this.setAutoBuild(projectInfo.autoBuild) || changed;
            }
            // note oldState can be null if this is the first time update is being invoked.
            const oldState = this._state;
            this._state = new ProjectState_1.default(projectInfo, oldState != null ? oldState : undefined);
            if (!this._state.equals(oldState)) {
                changed = true;
                const startModeMsg = projectInfo.startMode == null ? "" : `, startMode=${projectInfo.startMode}`;
                Logger_1.default.d(`${this.name} went from ${oldState} to ${this._state}${startModeMsg}`);
                // Check if the project was just enabled or disabled
                if (oldState != null) {
                    if (oldState.isEnabled && !this._state.isEnabled) {
                        this.onDisable();
                    }
                    else if (!oldState.isEnabled && this._state.isEnabled) {
                        this.onEnable();
                    }
                }
            }
            const ports = projectInfo.ports;
            if (ports != null) {
                changed = this.updatePorts(ports) || changed;
            }
            else if (this._state.isStarted) {
                Logger_1.default.e("No ports were provided for an app that is supposed to be started");
            }
            if (this.pendingRestart != null) {
                this.pendingRestart.onStateChange(this.state.appState);
            }
            // Logger.log(`${this.name} has a new status:`, this._state);
            if (changed) {
                // Log.d(`${this.name} has changed`);
                this.onChange();
            }
            return this._state;
        };
        Logger_1.default.d("Creating project from info:", projectInfo);
        this.name = projectInfo.name;
        this.id = projectInfo.projectID;
        // should use projectType not buildType but it's missing sometimes
        // https://github.ibm.com/dev-ex/portal/issues/520
        this.type = new ProjectType_1.default(projectInfo.buildType, projectInfo.language);
        this.localPath = vscode.Uri.file(MCUtil.appendPathWithoutDupe(connection.workspacePath.fsPath, vscode.Uri.file(projectInfo.locOnDisk).fsPath));
        this._contextRoot = projectInfo.contextroot || "";
        // These will be overridden by the call to update(), but we set them here too so the compiler can see they're always set.
        this._autoBuildEnabled = projectInfo.autoBuild;
        // lastbuild is a number
        this._lastBuild = new Date(projectInfo.lastbuild);
        // appImageLastBuild is a string
        this._lastImgBuild = new Date(Number(projectInfo.appImgLastBuild));
        this._ports = {
            appPort: undefined,
            debugPort: undefined,
            internalPort: undefined,
            internalDebugPort: undefined,
        };
        this._state = this.update(projectInfo);
        // QuickPickItem
        this.label = translator_1.default.t(STRING_NS, "quickPickLabel", { projectName: this.name, projectType: this.type.type });
        // this.detail = this.id;
        if (this.connection.is1905OrNewer()) {
            this.logManager = new MCLogManager_1.default(this);
        }
        else {
            this.logManager = new MCLogManager_Old_1.default(this.connection);
        }
        Logger_1.default.i(`Created project ${this.name}:`, this);
    }
    // description used by QuickPickItem
    get description() {
        const appUrl = this.appBaseUrl;
        if (appUrl != null) {
            return appUrl.toString();
        }
        else {
            return translator_1.default.t(STRING_NS, "quickPickNotRunning");
        }
    }
    /**
     * Call when this project's mutable fields change
     * to update the tree view and project info pages.
     */
    onChange() {
        this.connection.onChange(this);
        this.tryRefreshProjectInfoPage();
    }
    /**
     * Update this project's port fields. Does not call onChange().
     * @param ports - Ports object from a Microclimate socket event or Project info
     * @returns true if at least one port was changed
     */
    updatePorts(ports) {
        let changed = false;
        changed = this.setPort(ports.exposedPort, "appPort");
        changed = this.setPort(ports.exposedDebugPort, "debugPort") || changed;
        changed = this.setPort(ports.internalPort, "internalPort") || changed;
        changed = this.setPort(ports.internalDebugPort, "internalDebugPort") || changed;
        return changed;
    }
    onSettingsChangedEvent(event) {
        Logger_1.default.d("project settings changed " + this.name, event);
        if (event.status !== SocketEvents_1.default.STATUS_SUCCESS) {
            let errMsg = "Project settings update failed";
            Logger_1.default.e(errMsg, event.error);
            if (event.error) {
                errMsg += " " + event.error;
            }
            vscode.window.showErrorMessage(errMsg);
            return;
        }
        // Only one of contextroot, app port, or debug port should be set
        // but there's no reason to treat it differently if multiple are set
        let changed = false;
        if (event.contextRoot) {
            let newContextRoot = event.contextRoot;
            // Remove leading / if present
            if (newContextRoot.startsWith("/")) {
                newContextRoot = newContextRoot.substring(1, newContextRoot.length);
            }
            this._contextRoot = event.contextRoot;
            Logger_1.default.i("ContextRoot now " + this._contextRoot);
            changed = true;
        }
        if (event.ports) {
            if (event.ports.internalPort) {
                changed = this.setPort(event.ports.internalPort, "internalPort");
            }
            else if (event.ports.internalDebugPort) {
                changed = this.setPort(event.ports.internalDebugPort, "internalDebugPort");
            }
            else {
                Logger_1.default.e("Received unexpected ports response:", event.ports);
            }
        }
        if (changed) {
            this.onChange();
        }
    }
    doRestart(mode) {
        if (this.pendingRestart != null) {
            // should be prevented by the RestartProjectCommand
            Logger_1.default.e(this.name + ": doRestart called when already restarting");
            return false;
        }
        this.pendingRestart = new ProjectPendingRestart_1.default(this, mode, this.RESTART_TIMEOUT);
        return true;
    }
    onRestartFinish() {
        Logger_1.default.d(this.name + ": onRestartFinish");
        this.pendingRestart = undefined;
    }
    /**
     * Validate the restart event. If it succeeded, update ports.
     * Notifies the pendingRestart.
     */
    onRestartEvent(event) {
        let success;
        let errMsg;
        if (this.pendingRestart == null) {
            Logger_1.default.e(this.name + ": received restart event without a pending restart", event);
            return;
        }
        if (SocketEvents_1.default.STATUS_SUCCESS !== event.status) {
            Logger_1.default.e(`${this.name}: Restart failed, response is`, event);
            errMsg = translator_1.default.t(StringNamespaces_1.default.DEFAULT, "genericErrorProjectRestart", { thisName: this.name });
            if (event.errorMsg != null) {
                errMsg = event.errorMsg;
            }
            success = false;
        }
        else if (event.ports == null || event.startMode == null || !StartModes_1.default.allStartModes().includes(event.startMode)) {
            // If the status is "success" (as we just checked), these must all be set and valid
            errMsg = translator_1.default.t(StringNamespaces_1.default.DEFAULT, "genericErrorProjectRestart", { thisName: this.name });
            Logger_1.default.e(errMsg + ", payload:", event);
            success = false;
        }
        else {
            Logger_1.default.d("Restart event is valid");
            this.updatePorts(event.ports);
            this.onChange();
            success = true;
        }
        this.pendingRestart.onReceiveRestartEvent(success, errMsg);
    }
    onConnectionReconnect() {
        this.logManager.onReconnectOrEnable();
    }
    onConnectionDisconnect() {
        if (this.pendingRestart != null) {
            this.pendingRestart.onDisconnectOrDisable(true);
        }
        this.logManager.onDisconnectOrDisable(true);
    }
    onEnable() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.i(`${this.name} has been enabled`);
            this.logManager.onReconnectOrEnable();
        });
    }
    onDisable() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.i(`${this.name} has been disabled`);
            if (this.pendingRestart != null) {
                this.pendingRestart.onDisconnectOrDisable(false);
            }
            // this.logManager.destroyAllLogs();
            this.logManager.onDisconnectOrDisable(false);
        });
    }
    dispose() {
        return __awaiter(this, void 0, void 0, function* () {
            return Promise.all([
                this.clearValidationErrors(),
                this.logManager.destroyAllLogs(this.id),
                this.activeProjectInfo != null ? this.activeProjectInfo.dispose() : Promise.resolve(),
            ])
                .then(() => {
                this.connection.onChange(this);
                Promise.resolve();
            });
        });
    }
    /**
     * Call when this project is deleted in Microclimate
     */
    onDelete() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.i(`${this.name} was deleted`);
            vscode.window.showInformationMessage(translator_1.default.t(STRING_NS, "onDeletion", { projectName: this.name }));
            DebugUtils_1.default.removeDebugLaunchConfigFor(this);
            yield this.dispose();
        });
    }
    /**
     * Clear all diagnostics for this project's path
     */
    clearValidationErrors() {
        return __awaiter(this, void 0, void 0, function* () {
            Project.diagnostics.delete(this.localPath);
        });
    }
    ///// ProjectOverview
    /**
     * To be called when the user tries to open this project's Project Info page.
     *
     * If the user already has a Project Info page open for this project, returns the existing page.
     * In this case, the webview should be re-revealed, but a new one should not be created.
     * If the user does not already have an info page open for this project, returns undefined,
     * and sets the given webview to be this project's project info panel.
     */
    onOpenProjectInfo(wvPanel) {
        if (this.activeProjectInfo != null) {
            return this.activeProjectInfo;
        }
        // Log.d(`Info opened for project ${this.name}`);
        this.activeProjectInfo = wvPanel;
        return undefined;
    }
    closeProjectInfo() {
        if (this.activeProjectInfo != null) {
            this.activeProjectInfo.dispose();
            this.activeProjectInfo = undefined;
        }
    }
    tryRefreshProjectInfoPage() {
        if (this.activeProjectInfo != null) {
            // Log.d("Refreshing projectinfo");
            ProjectOverviewPage_1.refreshProjectOverview(this.activeProjectInfo, this);
        }
    }
    ///// Getters
    get isRestarting() {
        return this.pendingRestart != null;
    }
    get containerID() {
        return this._containerID;
    }
    get contextRoot() {
        return this._contextRoot;
    }
    get ports() {
        return this._ports;
    }
    get autoBuildEnabled() {
        return this._autoBuildEnabled;
    }
    get state() {
        return this._state;
    }
    get appBaseUrl() {
        if (this._ports.appPort == null || isNaN(this._ports.appPort)) {
            // app is stopped, disabled, etc.
            return undefined;
        }
        return this.connection.mcUri.with({
            authority: `${this.connection.host}:${this._ports.appPort}`,
            path: this._contextRoot
        });
    }
    get debugUrl() {
        if (this._ports.debugPort == null || isNaN(this._ports.debugPort)) {
            return undefined;
        }
        return this.connection.host + ":" + this._ports.debugPort; // non-nls
    }
    get lastBuild() {
        return this._lastBuild;
    }
    get lastImgBuild() {
        return this._lastImgBuild;
    }
    get hasContextRoot() {
        return this._contextRoot != null && this._contextRoot.length > 0 && this._contextRoot !== "/";
    }
    ///// Setters
    /**
     * Set one of this project's Port fields.
     * @param newPort Can be undefined if the caller wishes to "unset" the port (ie, because the app is stopping)
     * @returns true if at least one port was changed.
     */
    setPort(newPort, portType) {
        if (newPort === "") {
            newPort = undefined;
        }
        const newPortNumber = Number(newPort);
        const currentPort = this._ports[portType];
        if (newPort && !MCUtil.isGoodPort(newPortNumber)) {
            Logger_1.default.w(`Invalid ${portType} port ${newPort} given to project ${this.name}, ignoring it`);
            return false;
        }
        else if (currentPort !== newPortNumber) {
            if (isNaN(newPortNumber)) {
                this._ports[portType] = undefined;
                // Log.d(`Unset ${portType} for ${this.name}`);
            }
            else if (newPortNumber !== currentPort) {
                Logger_1.default.d(`New ${portType} for ${this.name} is ${newPortNumber}`);
                this._ports[portType] = newPortNumber;
            }
            // the third case is that (the new port === the old port) and neither are null - we don't log anything in this case.
            return true;
        }
        // Log.d(`${portType} port is already ${currentPort}`);
        return false;
    }
    setContainerID(newContainerID) {
        const oldContainerID = this._containerID;
        this._containerID = newContainerID;
        const changed = this._containerID !== oldContainerID;
        if (changed) {
            const asStr = this._containerID == null ? "undefined" : this._containerID.substring(0, 8);
            if (asStr.length === 0) {
                Logger_1.default.w(`Empty containerID for ${this.name}`);
            }
            Logger_1.default.d(`New containerID for ${this.name} is ${asStr}`);
        }
        return changed;
    }
    setLastBuild(newLastBuild) {
        if (newLastBuild == null) {
            return false;
        }
        const oldlastBuild = this._lastBuild;
        this._lastBuild = new Date(newLastBuild);
        const changed = this._lastBuild !== oldlastBuild;
        if (changed) {
            // Log.d(`New lastBuild for ${this.name} is ${this._lastBuild}`);
        }
        return changed;
    }
    setLastImgBuild(newLastImgBuild) {
        if (newLastImgBuild == null) {
            return false;
        }
        const oldlastImgBuild = this._lastImgBuild;
        this._lastImgBuild = new Date(newLastImgBuild);
        const changed = this._lastImgBuild !== oldlastImgBuild;
        if (changed) {
            // Log.d(`New lastImgBuild for ${this.name} is ${this._lastImgBuild}`);
        }
        return changed;
    }
    setAutoBuild(newAutoBuild) {
        if (newAutoBuild == null) {
            return false;
        }
        const oldAutoBuild = this._autoBuildEnabled;
        this._autoBuildEnabled = newAutoBuild;
        const changed = this._autoBuildEnabled !== oldAutoBuild;
        if (changed) {
            // onChange has to be invoked explicitly because this function can be called outside of update()
            Logger_1.default.d(`New autoBuild for ${this.name} is ${this._autoBuildEnabled}`);
            this.onChange();
        }
        return changed;
    }
}
Project.diagnostics = vscode.languages.createDiagnosticCollection("Microclimate");
exports.default = Project;
//# sourceMappingURL=Project.js.map