"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Resources_1 = require("../../constants/Resources");
const MCUtil = require("../../MCUtil");
// This file does have a bunch of strings that should be translated,
// but the stringfinder is not smart enough to pick them out from the regular html strings. So, do this file by hand.
// non-nls-file
/**
 * These are the messages the WebView can send back to its creator in ProjectInfoCmd
 */
var Messages;
(function (Messages) {
    Messages["BUILD"] = "build";
    Messages["TOGGLE_AUTOBUILD"] = "toggleAutoBuild";
    Messages["OPEN"] = "open";
    Messages["DELETE"] = "delete";
    Messages["TOGGLE_ENABLEMENT"] = "toggleEnablement";
    Messages["EDIT"] = "edit";
})(Messages = exports.Messages || (exports.Messages = {}));
var Openable;
(function (Openable) {
    Openable["WEB"] = "web";
    Openable["FILE"] = "file";
    Openable["FOLDER"] = "folder";
})(Openable = exports.Openable || (exports.Openable = {}));
var Editable;
(function (Editable) {
    Editable["CONTEXT_ROOT"] = "context-root";
    Editable["APP_PORT"] = "app-port";
    Editable["DEBUG_PORT"] = "debug-port";
    Editable["DISABLED"] = "disabled";
})(Editable = exports.Editable || (exports.Editable = {}));
function refreshProjectOverview(webviewPanel, project) {
    webviewPanel.webview.html = generateHtml(project);
}
exports.refreshProjectOverview = refreshProjectOverview;
function generateHtml(project) {
    const emptyRow = `
    <tr>
        <td>&nbsp;</td>
        <td>&nbsp;</td>
    </tr>
    `;
    const notAvailable = "Not available";
    const notRunning = "Not running";
    const notDebugging = "Not debugging";
    const supportsEditableSettings = project.connection.is1905OrNewer();
    return `
        <!DOCTYPE html>

        <html>
        <head>
            <meta charset="UTF-8">
            <!--meta http-equiv="Content-Security-Policy" content="default-src 'self' ;"-->
            <meta name="viewport" content="width=device-width, initial-scale=1.0">

            <link rel="stylesheet" href="${getStylesheetPath()}"/>
        </head>
        <body>

        <div id="main">
            <div id="top-section">
                <img id="mc-icon" width="30px" src="${getIcon(Resources_1.default.Icons.Microclimate)}"/>
                <h2>Project ${project.name}</h2>
                <input id="build-btn" type="button" value="Build"
                    onclick="${project.state.isEnabled ? `sendMsg('${Messages.BUILD}')` : ""}"
                    class="btn ${project.state.isEnabled ? "" : "btn-disabled"}"/>
            </div>

            <table>
                <!--${buildRow("Name", project.name)}-->
                ${buildRow("Type", project.type.toString())}
                <!--${buildRow("Microclimate URL", project.connection.toString())}-->
                ${buildRow("Project ID", project.id)}
                ${buildRow("Container ID", normalize(project.containerID, notAvailable, 32))}
                ${buildRow("Location on Disk", project.localPath.fsPath, Openable.FOLDER)}
                <tr>
                    <td class="info-label">Auto build:</td>
                    <td>
                        <input id="auto-build-toggle" type="checkbox" class="btn"
                            onclick="sendMsg('${Messages.TOGGLE_AUTOBUILD}')"
                            ${project.autoBuildEnabled ? "checked" : ""}
                            ${project.state.isEnabled ? " " : " disabled"}
                        />
                    </td>
                </tr>
                ${emptyRow}
                ${buildRow("Application Status", project.state.appState)}
                ${buildRow("Build Status", normalize(project.state.getBuildString(), notAvailable))}
                ${emptyRow}
                ${buildRow("Last Image Build", normalizeDate(project.lastImgBuild, notAvailable))}
                ${buildRow("Last Build", normalizeDate(project.lastBuild, notAvailable))}
            </table>

            <!-- Separate fixed table for the lower part so that the Edit buttons line up in their own column,
                but also don't appear too far to the right -->
            <table class="fixed-table">
                ${emptyRow}
                ${buildRow("Exposed App Port", normalize(project.ports.appPort, notRunning))}
                ${buildRow("Internal App Port", normalize(project.ports.internalPort, notAvailable), undefined, supportsEditableSettings ? Editable.APP_PORT : Editable.DISABLED)}
                ${buildRow("Application Endpoint", normalize(project.appBaseUrl, notRunning), (project.appBaseUrl != null ? Openable.WEB : undefined), supportsEditableSettings ? Editable.CONTEXT_ROOT : Editable.DISABLED)}
                ${emptyRow}
                ${buildRow("Exposed Debug Port", normalize(project.ports.debugPort, notDebugging))}
                ${buildRow("Internal Debug Port", normalize(project.ports.internalDebugPort, notAvailable), undefined, supportsEditableSettings ? Editable.DEBUG_PORT : Editable.DISABLED)}
                ${buildRow("Debug URL", normalize(project.debugUrl, notDebugging))}
            </table>

            <div id="bottom-section">
                <input id="delete-btn" type="button" onclick="sendMsg('${Messages.DELETE}')" class="btn" value="Delete project"/>
                <input id="enablement-btn" type="button" onclick="sendMsg('${Messages.TOGGLE_ENABLEMENT}')" class="btn"
                    value="${(project.state.isEnabled ? "Disable" : "Enable") + " project"}"/>
            </div>
        </div>

        <script type="text/javascript">
            const vscode = acquireVsCodeApi();

            function vscOpen(element, type) {
                sendMsg("${Messages.OPEN}", { type: type, value: element.textContent });
            }

            function sendMsg(type, data = undefined) {
                // See IWebViewMsg in ProjectOverviewCmd
                vscode.postMessage({ type: type, data: data });
            }
        </script>

        </body>
        </html>
    `;
}
exports.generateHtml = generateHtml;
const RESOURCE_SCHEME = "vscode-resource:";
function getStylesheetPath() {
    return RESOURCE_SCHEME + Resources_1.default.getCss("project-overview.css");
}
function getIcon(icon) {
    const iconPaths = Resources_1.default.getIconPaths(icon);
    // return RESOURCE_SCHEME + dark ? iconPaths.dark : iconPaths.light;
    return RESOURCE_SCHEME + iconPaths.dark;
}
function buildRow(label, data, openable, editable) {
    let secondColTdContents = "";
    let thirdColTdContents = "";
    if (openable) {
        secondColTdContents += `<a title="${label}" onclick="vscOpen(this, '${openable}')">${data}</a>`;
    }
    else {
        secondColTdContents = `${data}`;
    }
    if (editable) {
        const tooltip = `title=` + (editable === Editable.DISABLED ? `"Upgrade your Microclimate version to use this feature"` : "Edit");
        const cursor = editable === Editable.DISABLED ? `style="cursor: not-allowed;"` : "";
        const onClick = editable === Editable.DISABLED ? "" : `onclick="sendMsg('${Messages.EDIT}', { type: '${editable}' })"`;
        thirdColTdContents = `
            <img id="edit-${MCUtil.slug(label)}" class="edit-btn" ${tooltip} ${cursor} ${onClick}` +
            `src="${getIcon(Resources_1.default.Icons.Edit)}"/>
        `;
    }
    const secondTd = `<td title="${label}">${secondColTdContents}</td>`;
    const thirdTd = thirdColTdContents ? `<td>${thirdColTdContents}</td>` : "";
    return `
        <tr class="info-row">
            <td class="info-label">${label}:</td>
            ${secondTd}
            ${thirdTd}
        </tr>
    `;
}
/**
 * Convert `item` to a user-friendly string, or the fallback if `item` is undefined or invalid.
 */
function normalize(item, fallback, maxLength) {
    let result;
    if (item == null || item === "" || (typeof item === typeof 0 && isNaN(Number(item)))) {
        result = fallback;
    }
    else if (item instanceof vscode.Uri && item.scheme.includes("file")) {
        result = item.fsPath;
    }
    else {
        result = item.toString();
    }
    if (maxLength != null) {
        result = result.substring(0, maxLength);
    }
    return result;
}
function normalizeDate(d, fallback) {
    if (MCUtil.isGoodDate(d)) {
        let dateStr = d.toLocaleDateString();
        if (dateStr === (new Date()).toLocaleDateString()) {
            dateStr = "Today";
        }
        return `${dateStr} at ${d.toLocaleTimeString()}`;
    }
    else {
        return fallback;
    }
}
//# sourceMappingURL=ProjectOverviewPage.js.map