"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const request = require("request-promise-native");
const StartModes_1 = require("../../constants/StartModes");
const Logger_1 = require("../../Logger");
const StringNamespaces_1 = require("../../constants/strings/StringNamespaces");
const translator_1 = require("../../constants/strings/translator");
const MCUtil = require("../../MCUtil");
const Endpoints_1 = require("../../constants/Endpoints");
const STRING_NS = StringNamespaces_1.default.REQUESTS;
var Requester;
(function (Requester) {
    function requestProjectRestart(project, startMode) {
        return __awaiter(this, void 0, void 0, function* () {
            const body = {
                startMode: startMode.toString()
            };
            const restartMsg = translator_1.default.t(STRING_NS, "restartIntoMode", { startMode: StartModes_1.default.getUserFriendlyStartMode(startMode) });
            return doProjectRequest(project, Endpoints_1.ProjectEndpoints.RESTART_ACTION, body, request.post, restartMsg);
        });
    }
    Requester.requestProjectRestart = requestProjectRestart;
    function requestBuild(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const body = {
                action: "build" // non-nls
            };
            // return doProjectRequest(project, url, body, request.post, "Build");
            const buildMsg = translator_1.default.t(STRING_NS, "build");
            yield doProjectRequest(project, Endpoints_1.ProjectEndpoints.BUILD_ACTION, body, request.post, buildMsg);
            // This is a workaround for the Build action not refreshing validation state.
            // Will be fixed by https://github.ibm.com/dev-ex/iterative-dev/issues/530
            yield requestValidate(project, true);
        });
    }
    Requester.requestBuild = requestBuild;
    function requestToggleAutoBuild(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const newAutoBuild = !project.autoBuildEnabled;
            // user-friendly action
            const autoBuildMsgKey = newAutoBuild ? "autoBuildEnable" : "autoBuildDisable"; // non-nls
            const newAutoBuildUserStr = translator_1.default.t(STRING_NS, autoBuildMsgKey);
            // action we'll put into the request body   https://github.ibm.com/dev-ex/portal/wiki/API:-Build
            const newAutoBuildAction = newAutoBuild ? "enableautobuild" : "disableautobuild"; // non-nls
            const body = {
                action: newAutoBuildAction
            };
            const response = yield doProjectRequest(project, Endpoints_1.ProjectEndpoints.BUILD_ACTION, body, request.post, newAutoBuildUserStr);
            if (MCUtil.isGoodStatusCode(response.statusCode)) {
                Logger_1.default.d("Received good status from autoBuild request, new auto build is: " + newAutoBuild);
                project.setAutoBuild(newAutoBuild);
            }
        });
    }
    Requester.requestToggleAutoBuild = requestToggleAutoBuild;
    function requestToggleEnablement(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const newEnablement = !project.state.isEnabled;
            const newEnablementMsgKey = newEnablement ? "projectEnable" : "projectDisable"; // non-nls
            const newEnablementStr = translator_1.default.t(STRING_NS, newEnablementMsgKey);
            const endpoint = Endpoints_1.default.getEnablementAction(newEnablement);
            yield doProjectRequest(project, endpoint, {}, request.put, newEnablementStr);
        });
    }
    Requester.requestToggleEnablement = requestToggleEnablement;
    function requestValidate(project, silent) {
        return __awaiter(this, void 0, void 0, function* () {
            const [endpoint, body] = assembleValidateRequest(project, false);
            const userOperation = translator_1.default.t(StringNamespaces_1.default.CMD_MISC, "validate");
            yield doProjectRequest(project, endpoint, body, request.post, userOperation, silent);
        });
    }
    Requester.requestValidate = requestValidate;
    function requestGenerate(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const [endpoint, body] = assembleValidateRequest(project, true);
            const generateMsg = translator_1.default.t(STRING_NS, "generateMissingFiles");
            yield doProjectRequest(project, endpoint, body, request.post, generateMsg);
            // request a validate after the generate so that the validation errors go away faster
            yield requestValidate(project, true);
        });
    }
    Requester.requestGenerate = requestGenerate;
    // In 1901, the validation API changed
    const validationAPIChangeVersion = 1901;
    /**
     * Get the URL and request body for either a Validate or Generate request, they are very similar.
     */
    function assembleValidateRequest(project, generate) {
        const body = {
            projectType: project.type.internalType,
        };
        if (generate) {
            body.autoGenerate = true;
        }
        let endpoint;
        if (project.connection.version >= validationAPIChangeVersion) {
            endpoint = generate ? Endpoints_1.ProjectEndpoints.GENERATE : Endpoints_1.ProjectEndpoints.VALIDATE;
        }
        else {
            endpoint = generate ? Endpoints_1.MCEndpoints.GENERATE_OLD : Endpoints_1.MCEndpoints.VALIDATE_OLD;
            body.projectID = project.id;
        }
        return [endpoint, body];
    }
    function requestDelete(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const deleteMsg = translator_1.default.t(STRING_NS, "delete");
            yield doProjectRequest(project, Endpoints_1.ProjectEndpoints.NONE, {}, request.delete, deleteMsg);
        });
    }
    Requester.requestDelete = requestDelete;
    function requestSettingChange(project, settingName, settingKey, newValue, isNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            const updateMsg = `Update ${settingName}`;
            if (isNumber) {
                newValue = Number(newValue);
                if (isNaN(newValue)) {
                    throw new Error(`Failed to convert ${newValue} to number; ${settingName} must be a number.`);
                }
            }
            const body = {
                [settingKey]: newValue,
            };
            yield doProjectRequest(project, Endpoints_1.ProjectEndpoints.PROPERTES, body, request.post, updateMsg);
        });
    }
    Requester.requestSettingChange = requestSettingChange;
    function requestAvailableLogs(project) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!project.state.isEnabled) {
                // there are no logs available for disabled projects
                return {
                    build: [], app: []
                };
            }
            return (yield doProjectRequest(project, Endpoints_1.ProjectEndpoints.LOGS, {}, request.get, "available logs", true)).body;
        });
    }
    Requester.requestAvailableLogs = requestAvailableLogs;
    function requestToggleLogs(project, enable) {
        return __awaiter(this, void 0, void 0, function* () {
            const method = enable ? request.post : request.delete;
            yield doProjectRequest(project, Endpoints_1.ProjectEndpoints.LOGS, {}, method, `toggle logs ${enable ? "on" : "off"}`, true);
        });
    }
    Requester.requestToggleLogs = requestToggleLogs;
    function areMetricsAvailable(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield doProjectRequest(project, Endpoints_1.ProjectEndpoints.METRICS_STATUS, {}, request.get, "metrics status", true);
            const available = res.body.metricsAvailable;
            return available;
        });
    }
    Requester.areMetricsAvailable = areMetricsAvailable;
    /**
     * Perform a REST request of the type specified by `requestFunc` to the project endpoint for the given project.
     * Displays a message to the user that the request succeeded if userOperationName is not null.
     * Always displays a message to the user in the case of an error.
     * @param body - JSON request body for POST, PUT requests. Uses application/json content-type.
     * @param requestFunc - eg. request.get, request.post...
     * @param userOperationName - If `!silent`, a message will be displayed to the user that they have requested this operation on this project.
     * @param silent - If true, an info message will not be displayed when the request is initiated. Error messages are always displayed.
     */
    function doProjectRequest(project, endpoint, body, requestFunc, userOperationName, silent = false) {
        return __awaiter(this, void 0, void 0, function* () {
            let url;
            if (Endpoints_1.default.isProjectEndpoint(endpoint)) {
                url = Endpoints_1.default.resolveProjectEndpoint(project.connection, project.id, endpoint);
            }
            else {
                url = Endpoints_1.default.resolveMCEndpoint(project.connection, endpoint);
            }
            Logger_1.default.i(`Doing ${userOperationName} request to ${url}`);
            const options = {
                json: true,
                body,
                resolveWithFullResponse: true,
            };
            try {
                const result = yield requestFunc(url, options);
                Logger_1.default.d(`Response code ${result.statusCode} from ` +
                    `${userOperationName.toLowerCase()} request for ${project.name}`);
                if (!silent) {
                    vscode.window.showInformationMessage(translator_1.default.t(STRING_NS, "requestSuccess", { operationName: userOperationName, projectName: project.name }));
                }
                return result;
            }
            catch (err) {
                Logger_1.default.w(`Error doing ${userOperationName} project request for ${project.name}:`, err);
                vscode.window.showErrorMessage(translator_1.default.t(STRING_NS, "requestFail", { operationName: userOperationName, projectName: project.name, err: MCUtil.errToString(err) }));
                throw err;
            }
        });
    }
})(Requester || (Requester = {}));
exports.default = Requester;
//# sourceMappingURL=Requester.js.map