"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Project_1 = require("../microclimate/project/Project");
const Connection_1 = require("../microclimate/connection/Connection");
const translator_1 = require("../constants/strings/translator");
const Resources_1 = require("../constants/Resources");
const StringNamespaces_1 = require("../constants/strings/StringNamespaces");
const Logger_1 = require("../Logger");
const Commands_1 = require("../constants/Commands");
const ConnectionManager_1 = require("../microclimate/connection/ConnectionManager");
/**
 * All of these values must match the viewItem regexes in package.nls.json
 */
var TreeContextValues;
(function (TreeContextValues) {
    // base
    TreeContextValues["BASE"] = "ext.mc";
    TreeContextValues["NO_PROJECTS"] = "noProjects";
    // Connection
    // connection can only be one of these 3
    TreeContextValues["NO_CONNECTIONS"] = "noConnections";
    TreeContextValues["CONN_CONNECTED"] = "connection.connected";
    TreeContextValues["CONN_DISCONNECTED"] = "connection";
    // Project
    TreeContextValues["PROJ_BASE"] = "project";
    // (en|dis)abled are mutex
    TreeContextValues["PROJ_ENABLED"] = "enabled";
    TreeContextValues["PROJ_DISABLED"] = "disabled";
    // debuggable, started both imply enabled
    TreeContextValues["PROJ_DEBUGGABLE"] = "debuggable";
    TreeContextValues["PROJ_STARTED"] = "started";
    // auto build statuses are mutex
    TreeContextValues["PROJ_AUTOBUILD_ON"] = "autoBuildOn";
    TreeContextValues["PROJ_AUTOBUILD_OFF"] = "autoBuildOff";
})(TreeContextValues || (TreeContextValues = {}));
var TreeItemFactory;
(function (TreeItemFactory) {
    function getRootTreeItems() {
        if (ConnectionManager_1.default.instance.connections.length === 0) {
            return [{
                    label: "No connections (Click to connect)",
                    iconPath: Resources_1.default.getIconPaths(Resources_1.default.Icons.Microclimate),
                    contextValue: buildContextValue([TreeContextValues.NO_CONNECTIONS]),
                    command: {
                        command: Commands_1.default.NEW_DEFAULT_CONNECTION,
                        title: "",
                    },
                }];
        }
        return ConnectionManager_1.default.instance.connections;
    }
    TreeItemFactory.getRootTreeItems = getRootTreeItems;
    function toTreeItem(resource) {
        if (resource instanceof Project_1.default) {
            return getProjectTI(resource);
        }
        else if (resource instanceof Connection_1.default) {
            return getConnectionTI(resource);
        }
        else {
            // darn you, theia
            const errMsg = "Unexpected object cannot be converted to TreeItem";
            Logger_1.default.e(errMsg, resource);
            throw new Error(errMsg);
        }
    }
    TreeItemFactory.toTreeItem = toTreeItem;
    function getConnectionChildren(connection) {
        if (connection.isConnected) {
            if (connection.projects.length > 0) {
                return connection.projects.sort((a, b) => a.name.localeCompare(b.name));
            }
            else {
                return [{
                        label: translator_1.default.t(StringNamespaces_1.default.TREEVIEW, "noProjectsLabel"),
                        iconPath: Resources_1.default.getIconPaths(Resources_1.default.Icons.Error),
                        tooltip: "Click here to create a new project",
                        contextValue: buildContextValue([TreeContextValues.NO_PROJECTS]),
                        command: {
                            command: Commands_1.default.CREATE_MC_PROJECT,
                            title: "",
                            arguments: [connection]
                        }
                    }];
            }
        }
        else {
            return [{
                    label: translator_1.default.t(StringNamespaces_1.default.TREEVIEW, "disconnectedConnectionLabel"),
                    iconPath: Resources_1.default.getIconPaths(Resources_1.default.Icons.Disconnected),
                    contextValue: "nothing",
                }];
        }
    }
    TreeItemFactory.getConnectionChildren = getConnectionChildren;
})(TreeItemFactory || (TreeItemFactory = {}));
function getConnectionTI(connection) {
    return {
        label: translator_1.default.t(StringNamespaces_1.default.TREEVIEW, "connectionLabel", { uri: connection.mcUri }),
        collapsibleState: vscode.TreeItemCollapsibleState.Expanded,
        tooltip: `${connection.versionStr} • ${connection.workspacePath.fsPath}`,
        contextValue: getConnectionContext(connection),
        iconPath: Resources_1.default.getIconPaths(Resources_1.default.Icons.Microclimate),
    };
}
function getProjectTI(project) {
    return {
        label: translator_1.default.t(StringNamespaces_1.default.TREEVIEW, "projectLabel", { projectName: project.name, state: project.state.toString() }),
        collapsibleState: vscode.TreeItemCollapsibleState.None,
        tooltip: project.state.toString(),
        contextValue: getProjectContext(project),
        iconPath: project.type.icon,
        // command run on single-click (or double click - depends on a user setting - https://github.com/Microsoft/vscode/issues/39601)
        command: {
            command: Commands_1.default.VSC_REVEAL_EXPLORER,
            title: "",
            arguments: [project.localPath]
        },
    };
}
function getConnectionContext(connection) {
    let contextValue;
    if (connection.isConnected) {
        contextValue = TreeContextValues.CONN_CONNECTED;
    }
    else {
        contextValue = TreeContextValues.CONN_DISCONNECTED;
    }
    return buildContextValue([contextValue]);
}
function getProjectContext(project) {
    const contextValues = [TreeContextValues.PROJ_BASE];
    if (project.state.isEnabled) {
        contextValues.push(TreeContextValues.PROJ_ENABLED);
        if (project.state.isStarted) {
            contextValues.push(TreeContextValues.PROJ_STARTED);
        }
        if (project.state.isDebuggable) {
            contextValues.push(TreeContextValues.PROJ_DEBUGGABLE);
        }
    }
    else {
        contextValues.push(TreeContextValues.PROJ_DISABLED);
    }
    if (project.autoBuildEnabled) {
        contextValues.push(TreeContextValues.PROJ_AUTOBUILD_ON);
    }
    else {
        contextValues.push(TreeContextValues.PROJ_AUTOBUILD_OFF);
    }
    // The final result will look like eg: "ext.mc.project.enabled.autoBuildOn"
    return buildContextValue(contextValues);
}
// const CONTEXT_SEPARATOR = ".";
function buildContextValue(subvalues) {
    return [TreeContextValues.BASE, ...subvalues].join(".");
}
// export { TreeContextValues };
exports.default = TreeItemFactory;
//# sourceMappingURL=TreeItemFactory.js.map